/***************************************************************************
                          kbudgetdoc.cpp  -  description
                             -------------------
    begin                : Sat Aug 10 16:03:35 CST 2002
    copyright            : (C) 2002 by Richard Garand
    email                : richard@garandnet.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// include files for Qt
#include <qdir.h>
#include <qwidget.h>

// include files for KDE
#include <klocale.h>
#include <kmessagebox.h>
#include <kio/job.h>
#include <kio/netaccess.h>
#include <ktempfile.h>
#include <ksavefile.h>

// application specific includes
#include "kbudgetdoc.h"
#include "kbudget.h"
#include "kbudgetview.h"

// in case getAccount fails
Account blankAcct;
// in case getTransaction fails
Transaction blankTransaction;

QList<KBudgetView> *KBudgetDoc::pViewList = 0L;

KBudgetDoc::KBudgetDoc(QWidget *parent, const char *name) : QObject(parent, name), nextAccount(0),
  nextTransaction(0)
{
  if(!pViewList)
  {
    pViewList = new QList<KBudgetView>();
  }

  pViewList->setAutoDelete(true);
}

KBudgetDoc::~KBudgetDoc()
{
}

void KBudgetDoc::addView(KBudgetView *view)
{
  pViewList->append(view);
}

void KBudgetDoc::removeView(KBudgetView *view)
{
  pViewList->remove(view);
}
void KBudgetDoc::setURL(const KURL &url)
{
  doc_url=url;
}

void KBudgetDoc::setModified(bool status)
{
  modified = status;
}

const KURL& KBudgetDoc::URL() const
{
  return doc_url;
}

void KBudgetDoc::slotUpdateAllViews(KBudgetView *sender)
{
  KBudgetView *w;
  if(pViewList)
  {
    for(w=pViewList->first(); w!=0; w=pViewList->next())
    {
      if(w!=sender)
        w->repaint();
    }
  }

}

bool KBudgetDoc::saveModified()
{
  bool completed=true;

  if(modified)
  {
    KBudgetApp *win=(KBudgetApp *) parent();
    int want_save = KMessageBox::warningYesNoCancel(win,
                                         i18n("The current file has been modified.\n"
                                              "Do you want to save it?"),
                                         i18n("Warning"));
    switch(want_save)
    {
      case KMessageBox::Yes:
           if (doc_url.fileName() == i18n("Untitled"))
           {
             win->slotFileSaveAs();
           }
           else
           {
             saveDocument(URL());
       	   };

       	   deleteContents();
           completed=true;
           break;

      case KMessageBox::No:
           modified=false;
           emit saveableStatus(true);
           deleteContents();
           completed=true;
           break;

      case KMessageBox::Cancel:
           completed=false;
           break;

      default:
           completed=false;
           break;
    }
  }

  return completed;
}

void KBudgetDoc::closeDocument()
{
  deleteContents();
}

bool KBudgetDoc::newDocument()
{
  closeDocument();
  emit accountAddDel();
  doc_url.setFileName(i18n("Untitled"));
  modified=false;
  emit saveableStatus(true);

  return true;
}
#include <stdio.h>
bool KBudgetDoc::openDocument(const KURL& url, const char *format /*=0*/)
{
  bool done = false;

  QString tmpfile;
  if ( !url.isLocalFile() )
    KIO::NetAccess::download( url, tmpfile );
  else
    tmpfile = url.path();

  QFile ifile(tmpfile);
  if ( !ifile.open(IO_ReadOnly) )
  	return false;
  if ( !ifile.size() )
    done = true;

  QTextStream inf(&ifile);
  QString type, name;
  inf >> type;
  name = inf.readLine().stripWhiteSpace();
  if ( !done && (type != "KBudget" || name != "doc 0.1") ) {
    KMessageBox mb;
    QString error = i18n("Error: %1 isn't a valid KBudget file");
    error.arg(tmpfile);
    mb.error(0, error, i18n("Couldn't open file"));
    done = true;
  }
  if ( !done )
	closeDocument();

  while ( !done ) {
    inf >> type;
    if ( type == "Transaction" ) {
      int id, from, to, year, month, day;
      float value;
      inf >> id >> from >> to >> value >> year >> month >> day;
      name = inf.readLine().stripWhiteSpace();
      if ( transactions.find(id) != transactions.end() ) {
        printf("Duplicate transaction %i\n", id);
      } else {
        transactions[id] = Transaction(this, id, from, to, value, QDate(year, month, day), name);
      }
      if ( id >= nextTransaction )
        nextTransaction = id + 1;
    } else if ( type == "Account" ) {
      int id, type;
      float startBalance;
      inf >> id >> type >> startBalance;
      name = inf.readLine().stripWhiteSpace();
      if ( accounts[type].find(id) != accounts[type].end() ) {
        printf("Duplicate account %i\n", id);
      } else {
        accounts[type][id] = Account(this, id, (Account::types)type, name, startBalance);
      }
      if ( id >= nextAccount )
        nextAccount = id + 1;
    }
    done = inf.atEnd();
  }
  ifile.close();

  if ( !url.isLocalFile() )
	KIO::NetAccess::removeTempFile( tmpfile );

  QValueList<int> txs = transactions.keys();
  for ( unsigned int i = 0; i < txs.size(); i++ ) {
    if ( isValidAccount(transactions[txs[i]].from()) )
      getAccount(transactions[txs[i]].from()).addTransaction(txs[i]);
    if ( isValidAccount(transactions[txs[i]].to()) )
      getAccount(transactions[txs[i]].to()).addTransaction(txs[i]);
  }
  for ( int i = 0; i < 3; i++ )
    for ( QMap<int,Account>::const_iterator ac = accounts[i].begin(); ac != accounts[i].end(); ac++ )
      (*ac).dump();

  emit accountAddDel();
  
  modified=false;
  doc_url = url;
  emit saveableStatus(false);
  return true;
}

bool KBudgetDoc::saveDocument(const KURL& url, const char *format /*=0*/)
{
  if ( url.isMalformed() ) {
    KMessageBox::sorry(0, i18n("%1 is a malformed URL.").arg(url.prettyURL()));
    return false;
  }

  if ( url.isLocalFile() ) {
    if ( url != doc_url && QFile::exists(url.path()) ) {
      if ( KMessageBox::warningYesNo(0, i18n("That file already exists. Would you like to overwrite the old copy?"))\
        != KMessageBox::Yes)
        return false;
    }
    return writeDocument(url.path());
  }

  KTempFile tf;
  tf.setAutoDelete(true);
  return writeDocument(tf.name());
}

void KBudgetDoc::deleteContents()
{
  for ( int i = 0; i < 3; i++ )
    accounts[i].clear();
  transactions.clear();
  nextAccount = 0;
  nextTransaction = 0;
}

Account& KBudgetDoc::getAccount(int id)
{
  for ( int i = 0; i < 3; i++ )
	  if ( accounts[i].find(id) != accounts[i].end() )
	    return accounts[i][id];
  return blankAcct;
}

QValueList<Account> KBudgetDoc::getAccounts(Account::types type, QDate from, QDate to) const
{
  QValueList<Account> retList;
  QMap<int,Account>::const_iterator account = accounts[type].begin();
  for ( ; account != accounts[type].end(); account++ ) {
//    (*account).dump();
    Account temp = (*account).clone(from, to);
    temp.dump();
    retList.push_back(temp);
  }
  return retList;
}

QValueList<Account> KBudgetDoc::getAllAccounts() const
{
  QValueList<Account> ret = accounts[0].values();
  ret += accounts[1].values();
  ret += accounts[2].values();
  return ret;
}

Transaction& KBudgetDoc::getTransaction(int id)
{
  if ( transactions.find(id) != transactions.end() )
    return transactions[id];
  return blankTransaction;
}

int KBudgetDoc::newAccount(Account::types type, QString name, float initialBalance)
{
  accounts[type].insert(nextAccount, Account(this, nextAccount, type, name, initialBalance));
  nextAccount++;
  
  emit accountAddDel();
  modified=true;
  emit saveableStatus(true);
  return nextAccount - 1;
}

bool KBudgetDoc::isValidAccount(int id)
{
  for ( int i = 0; i < 3; i++ )
    if ( accounts[i].find(id) != accounts[i].end() )
      return true;
  return false;
}

int KBudgetDoc::newTransaction(int from, int to, float value, QDate date, QString description)
{
  transactions.insert(nextTransaction, Transaction(this, nextTransaction, from, to, value, date, description));
  getAccount(from).addTransaction(nextTransaction);
  getAccount(to).addTransaction(nextTransaction);
  nextTransaction++;

  emit accountChanged(from);
  emit accountChanged(to);
  modified=true;
  emit saveableStatus(true);
  return nextTransaction - 1;
}

void KBudgetDoc::signalAcctChanged(int acct)
{
  modified=true;
  emit saveableStatus(true);
  emit accountChanged(acct);
}

bool KBudgetDoc::writeDocument(QString file)
{
  QFileInfo info(file);
  if ( info.isDir() ) {
    KMessageBox::sorry(0, i18n("You selected a directory!"));
    return false;
  }

  KSaveFile ofile(file, 0600);
  if ( ofile.status() ) {
    KMessageBox::sorry(0, i18n("Couldn't open file for writing"));
    ofile.abort();
    return false;
  }

  QTextStream &outf = *ofile.textStream();
  outf << "KBudget doc 0.1\n";

  QMap<int,Transaction>::const_iterator t = transactions.begin();
  for ( ; t != transactions.end(); t++ ) {
    QDate date = (*t).date();
    outf << QString("Transaction %1 %2 %3 %4 %5 %6 %7 %8\n")\
      .arg((*t).id()).arg((*t).from()).arg((*t).to()).arg((*t).value())\
      .arg(date.year()).arg(date.month()).arg(date.day()).arg((*t).description());
  }

  QMap<int,Account>::const_iterator acct;
  for ( int i = 0; i < 3; i++ )
    for ( acct = accounts[i].begin(); acct != accounts[i].end(); acct++ )
      outf << QString("Account %1 %2 %3 %4\n").arg((*acct).id()).arg(i)\
        .arg((*acct).startingBalance()).arg((*acct).name());

  if ( ofile.status() ) {
    KMessageBox::error(0, i18n("Error while writing file; file was not saved"));
    ofile.abort();
    return false;
  }

  ofile.close();
  if ( ofile.status() ) {
    KMessageBox::sorry(0, i18n("Error after saving file; data may not have been saved"));
    return false;
  }

  modified=false;
  emit saveableStatus(false);
  return true;
}
